@@ -1,4 +1,4 @@
-## complexity.r
+## complexity.R
 ##   - Complexity measures for GP individuals and estimators of GP search space size
 ##
 ## RGP - a GP system for R
@@ -20,6 +20,8 @@
 ##' \code{exprVisitationLength} returns the visitation length of the tree of an R expression.
 ##' The visitation length is the total number of nodes in all possible subtrees of a tree.
 ##' \code{funcVisitationLength} returns the visitation length of the body expression tree of an R function.
+##' \code{fastExprVisitationLength} and \code{fastFuncVisitationLength} are variants written in optimized
+##' C code.
 ##' The visitation length can be interpreted as the size of the expression obtained by substituting all
 ##' inner functions by their function bodies (see "Crossover Bias in Genetic Programming", Maarten Keijzer
 ##' and James Foster).
@@ -27,6 +29,7 @@
 ##' @param expr An R expression.
 ##' @param func An R function.
 ##' @param predicate An R predicate (function with range type \code{logical}).
+##' @param intermediateResults Whether to return complexity measures for all subtrees also.
 ##'
 ##' @rdname expressionComplexityMeasures
 ##' @export
@@ -79,25 +82,44 @@ funcCount <- function(func, predicate = function(node) TRUE) exprCount(body(func
 
 ##' @rdname expressionComplexityMeasures
 ##' @export
-exprVisitationLength <- function(expr)
-  exprVisitationLengthRecursive(expr)[[2]]
-
-##' @rdname expressionComplexityMeasures
-exprVisitationLengthRecursive <- function(expr)
-  ## The visitaion length of a tree T is the sum of the number of nodes of all subtrees of T...
-  if (is.call(expr)) {
-    childrenResults <- lapply(rest(expr), exprVisitationLengthRecursive)
-    childrenSums <- Reduce(function(a,b) c(a[1] + b[1], a[2] + b[2]), childrenResults, c(0,0))
-    childrenSizesSum <- childrenSums[1]
-    childrenVisitationLengthsSum <- childrenSums[2]
-    thisTreeSize <- 1 + childrenSizesSum
-    thisTreeVisitationLength <- thisTreeSize + childrenVisitationLengthsSum
-    c(thisTreeSize, thisTreeVisitationLength)
-  } else c(1, 1)
+exprVisitationLength <- function(expr, intermediateResults = FALSE) {
+  results <- c()
+  exprVisitationLengthRecursive <- function(expr) {
+    ## The visitation length of a tree T is the sum of the number of nodes of all subtrees of T...
+    if (is.call(expr)) {
+      childrenResults <- lapply(rest(expr), exprVisitationLengthRecursive)
+      childrenSums <- Reduce(function(a,b) c(a[1] + b[1], a[2] + b[2]), childrenResults, c(0,0))
+      childrenSizesSum <- childrenSums[1]
+      childrenVisitationLengthsSum <- childrenSums[2]
+      thisTreeSize <- 1 + childrenSizesSum
+      thisTreeVisitationLength <- thisTreeSize + childrenVisitationLengthsSum
+      if (intermediateResults) results <<- c(thisTreeVisitationLength, results)
+      c(thisTreeSize, thisTreeVisitationLength)
+    } else {
+      if (intermediateResults) results <<- c(1, results)
+      c(1, 1)
+    }
+  }
+  if (intermediateResults) {
+    exprVisitationLengthRecursive(expr)
+    results
+  } else exprVisitationLengthRecursive(expr)[[2]]
+}
 
 ##' @rdname expressionComplexityMeasures
 ##' @export
-funcVisitationLength <- function(func) exprVisitationLength(body(func))
+fastExprVisitationLength <- function(expr, intermediateResults = FALSE)
+  .Call("sexp_visitation_length_R", expr, intermediateResults) 
+
+##' @rdname expressionComplexityMeasures
+##' @export
+funcVisitationLength <- function(func, intermediateResults = FALSE)
+  exprVisitationLength(body(func), intermediateResults = intermediateResults)
+
+##' @rdname expressionComplexityMeasures
+##' @export
+fastFuncVisitationLength <- function(func, intermediateResults = FALSE)
+  .Call("func_visitation_length_R", func, intermediateResults) 
 
 ##' Upper bounds for expression tree search space sizes
 ##'
@@ -129,7 +151,7 @@ exprShapesOfDepth <- function(funcset, n)
     1 # there is only one expression shape of depth 1
   } else {
     exprShapesOfDepthNminusOne <- exprShapesOfDepth(funcset, n - 1)
-    uniqueArities <- unique(as.vector(Map(arity, funcset$all), mode = "integer"))
+    uniqueArities <- unique(funcset$arities)
     exprsWithFixedRootArity <- exprShapesOfDepthNminusOne ^ uniqueArities
     sum(exprsWithFixedRootArity)
   }
@@ -143,7 +165,7 @@ exprShapesOfMaxDepth <- function(funcset, n)
     1 # there is only one expression shape of max depth 1
   } else {
     exprShapesOfDepthNminusOne <- exprShapesOfMaxDepth(funcset, n - 1)
-    uniqueArities <- unique(as.vector(Map(arity, funcset$all), mode = "integer"))
+    uniqueArities <- unique(funcset$arities)
     exprsWithFixedRootArity <- exprShapesOfDepthNminusOne ^ uniqueArities
     sum(exprsWithFixedRootArity) + 1
   }
@@ -157,7 +179,7 @@ exprsOfDepth <- function(funcset, inset, n)
     1 + length(inset) # an expression of depth 1 is either a constant or an input variable
   } else {
     exprsOfDepthNminusOne <- exprsOfDepth(funcset, inset, n - 1)
-    arities <- as.vector(Map(arity, funcset$all), mode = "integer")
+    arities <- funcset$arities
     exprsWithFixedRoot <- exprsOfDepthNminusOne ^ arities
     sum(exprsWithFixedRoot)
   }
@@ -171,7 +193,7 @@ exprsOfMaxDepth <- function(funcset, inset, n)
     1 + length(inset) # an expression of depth 1 is either a constant or an input variable
   } else {
     exprsOfMaxDepthNminusOne <- exprsOfMaxDepth(funcset, inset, n - 1)
-    arities <- as.vector(Map(arity, funcset$all), mode = "integer")
+    arities <- funcset$arities
     exprsWithFixedRoot <- exprsOfMaxDepthNminusOne ^ arities
     sum(exprsWithFixedRoot) + 1 + length(inset)
   }
