@@ -1,4 +1,4 @@
-## expression_utils.r
+## expression_utils.R
 ##   - Utility functions for R expressions
 ##
 ## RGP - a GP system for R
@@ -18,6 +18,11 @@
 ##' for calls to \code{MapExpressionNodes}.
 ##  \code{FlattenExpression} returns a list of all nodes in an expression
 ##' \code{expr}.
+##  \code{subtrees} returns a list of all subtrees/function symbols/leafs of
+##' an expression \code{expr}.
+##  \code{expressionGraph} returns a graph representation of an expression
+##' \code{expr}, given as list of nodes and and list of vertices. Each
+##' vertex is represented as a pair of indices into the list of nodes.
 ##' \code{AllExpressionNodes} checks if all nodes in the tree of \code{expr}
 ##' satisfy the predicate \code{p} (\code{p} returns \code{TRUE} for every node).
 ##' This function short-cuts returning \code{FALSE} as soon as a node that
@@ -25,6 +30,9 @@
 ##' \code{AnyExpressionNode} checks if any node in the tree of \code{expr}
 ##' satisfies the predicate \code{p}. This function short-cuts returning
 ##' \code{TRUE} as soon as a node that satisfies \code{p} is encountered.
+##' \code{subtreeAt} returns the subtree at \code{index}.
+##' \code{replaceSubtreeAt} replaces the subtree at \code{index} with
+##' \code{replacement} and returns the result. 
 ##'
 ##' @param f The function to apply.
 ##' @param functions Whether to apply \code{f} to the function symbols
@@ -35,9 +43,12 @@
 ##'   Defaults to \code{TRUE}.
 ##' @param p The predicate to check.
 ##' @param expr The expression to transform.
+##' @param index An in-order subtree index starting from \code{0} (the root).
+##' @param replacement An expression. 
 ##' @return The transformed expression.
 ##'
 ##' @rdname expressionTransformation
+##' @export
 MapExpressionNodes <- function(f, expr, functions = TRUE, inners = FALSE, leafs = TRUE) {
   if (is.call(expr)) {
     oldfunc <- expr[[1]]
@@ -50,12 +61,15 @@ MapExpressionNodes <- function(f, expr, functions = TRUE, inners = FALSE, leafs
 }
 
 ##' @rdname expressionTransformation
+##' @export
 MapExpressionLeafs <- function(f, expr) MapExpressionNodes(f, expr, FALSE, FALSE, TRUE)
 
 ##' @rdname expressionTransformation
+##' @export
 MapExpressionSubtrees <- function(f, expr) MapExpressionNodes(f, expr, TRUE, TRUE, TRUE)
 
 ##' @rdname expressionTransformation
+##' @export
 FlattenExpression <- function(expr) {
   if (is.call(expr)) {
     func <- expr[[1]]
@@ -66,6 +80,52 @@ FlattenExpression <- function(expr) {
 }
 
 ##' @rdname expressionTransformation
+##' @export
+subtrees <- function(expr, functions = FALSE, inners = TRUE, leafs = TRUE) {
+  if (is.call(expr)) {
+    func <- expr[[1]]
+    if (functions && inners)
+      c(list(func), expr, Map(function(e) subtrees(e, functions, inners, leafs), expr[-1]), recursive = TRUE)
+    else if (inners)
+      c(expr, Map(function(e) subtrees(e, functions, inners, leafs), expr[-1]), recursive = TRUE)
+    else if (functions)
+      c(list(func), Map(function(e) subtrees(e, functions, inners, leafs), expr[-1]), recursive = TRUE)
+    else
+      c(Map(function(e) subtrees(e, functions, inners, leafs), expr[-1]), recursive = TRUE)
+  } else {
+    if (leafs) list(expr) else list()
+  }
+}
+
+##' @rdname expressionTransformation
+##' @export
+expressionGraph <- function(expr) {
+  currentNodeIndex <- 0 
+  expressionGraphRecursive <- function(expr) {
+    currentNodeIndex <<- currentNodeIndex + 1
+    if (is.call(expr)) {
+      children <- expr[-1]
+      nodes <- list()
+      vertices <- list()
+      currentRootIndex <- currentNodeIndex
+      for (i in 1:length(children)) {
+        currentChildIndex <- currentNodeIndex + 1
+        argumentGraph <- expressionGraphRecursive(children[[i]])
+        nodes <- c(nodes, argumentGraph$nodes)
+        vertices <- c(vertices, list(c(currentRootIndex, currentChildIndex)), argumentGraph$vertices)
+      }
+      list(nodes = c(expr, nodes), vertices = vertices)
+    } else {
+      nodes <- list(expr)
+      vertices <- list()
+      list(nodes = nodes, vertices = vertices)
+    }
+  }
+  expressionGraphRecursive(expr)
+}
+
+##' @rdname expressionTransformation
+##' @export
 AllExpressionNodes <- function(p, expr) {
   if (is.call(expr)) {
     if (!p(expr[[1]])) return(FALSE) # check function
@@ -81,6 +141,7 @@ AllExpressionNodes <- function(p, expr) {
 }
 
 ##' @rdname expressionTransformation
+##' @export
 AnyExpressionNode <- function(p, expr) {
   if (is.call(expr)) {
     if (p(expr[[1]])) return(TRUE) # check function
@@ -95,9 +156,46 @@ AnyExpressionNode <- function(p, expr) {
   }
 }
 
+##' @rdname expressionTransformation
+##' @export
+subtreeAt <- function(expr, index) subtreeAtRecursive(expr, index)
+
+subtreeAtRecursive <- function(expr, index, currentIndex = 0)
+  if (currentIndex == index) {
+    expr
+  } else if (is.call(expr)) { 
+    for (i in 2:length(expr)) {
+      result <- subtreeAtRecursive(expr[[i]], index, currentIndex + (i - 1))
+      if (!is.null(result)) {
+        return (result)
+      }
+    }
+    NULL # NULL := subtree not found
+  } else {
+    NULL # NULL := subtree not found
+  }
+
+##' @rdname expressionTransformation
+##' @export
+replaceSubtreeAt <- function(expr, index, replacement) replaceSubtreeAtRecursive(expr, index, replacement)
+
+replaceSubtreeAtRecursive <- function(expr, index, replacement, currentIndex = 0)
+  if (currentIndex == index) {
+    replacement 
+  } else if (is.call(expr)) { 
+    as.call(append(expr[[1]],
+                   Map(function(e, i) if (index == i) {
+                                        replacement
+                                      } else {
+                                        replaceSubtreeAtRecursive(e, index, replacement, currentIndex + i)
+                                      }, expr[-1], 1:(length(expr) - 1))))
+  } else {
+    expr 
+  }
+
 ##' Functions for decomposing and recombining R expressions
 ##'
-##' \code{subExpressions} returns a list of all subexpressions (subtrees) of an
+##' \code{subexpressions} returns a list of all subexpressions (subtrees) of an
 ##' expression \code{expr}.
 ##'
 ##' @param expr An R expression.
