@@ -1,4 +1,4 @@
-## recombination.r
+## recombination.R
 ##   - Functions for recombining GP individuals (e.g. by crossover)
 ##
 ## RGP - a GP system for R
@@ -7,9 +7,6 @@
 ## released under the GPL v2
 ##
 
-##' @include breeding.r
-NA
-
 ##' Select random childs or subtrees of an expression
 ##'
 ##' \code{randchild} returns a uniformly random direct child of an expression.
@@ -36,8 +33,13 @@ randsubtree <- function(expr, subtreeprob = 0.1)
 ##' \code{func2} (\code{expr2}) and return the resulting function (expression), i.e.
 ##' the modified \code{func1} (\code{expr1}).
 ##' \code{crossoverexpr} handles crossover of expressions instead of functions.
+##' \code{crossoverexprFast} is a fast (i.e. implemented in efficient C code)
+##' albeit less flexible variant of \code{crossoverexpr}.
 ##' \code{crossoverTyped} and \code{crossoverexprTyped} only exchage replace subtress
 ##' if the sTypes of their root nodes match.
+##' \code{crossoverTwoPoint} is a variant of \code{crossover} that swaps subtrees
+##' at uniform randomly selected points and returns both children.
+##' \code{crossoverexprTwoPoint} works analogously for expressions.
 ##'
 ##' All RGP recombination operators operating on functions have the S3 class
 ##' \code{c("recombinationOperator", "function")}.
@@ -51,7 +53,7 @@ randsubtree <- function(expr, subtreeprob = 0.1)
 ##' @param breedingFitness A breeding function. See the documentation for
 ##'   \code{\link{geneticProgramming}} for details.
 ##' @param breedingTries The number of breeding steps.
-##' @return The child function (expression).
+##' @return The child function (expression) or functions (expressions).
 ##'
 ##' @rdname expressionCrossover
 ##' @export
@@ -59,7 +61,7 @@ crossover <- function(func1, func2, crossoverprob = 0.1,
                       breedingFitness = function(individual) TRUE,
                       breedingTries = 50) {
   doCrossover <- function() {
-    child <- new.function() 
+    child <- new.function(envir = environment(func1)) 
     formals(child) <- formals(func1)
     body(child) <- crossoverexpr(body(func1), body(func2), crossoverprob)
     child
@@ -73,24 +75,39 @@ class(crossover) <- c("recombinationOperator", "function")
 crossoverexpr <- function(expr1, expr2, crossoverprob) {
   newexpr1 <- expr1
   if (is.call(expr1)) {
-    crossoverindex <- ceiling(runif(1, 1, length(expr1)))
+    crossoverindexA <- ceiling(runif(1, 1, length(expr1)))
     if (runif(1) <= crossoverprob) { # try to do crossover here
-      if (runif(1) > buildingBlockTag(newexpr1[[crossoverindex]]))
-        newexpr1[[crossoverindex]] <- randsubtree(expr2, crossoverprob)
+      if (runif(1) > buildingBlockTag(newexpr1[[crossoverindexA]]))
+        newexpr1[[crossoverindexA]] <- randsubtree(expr2, crossoverprob)
     } else { # try to do crossover somewhere below in this tree
-      newexpr1[[crossoverindex]] <- crossoverexpr(expr1[[crossoverindex]], expr2, crossoverprob)
+      newexpr1[[crossoverindexA]] <- crossoverexpr(expr1[[crossoverindexA]], expr2, crossoverprob)
     }
   }
   newexpr1
 }
 
+##' @rdname expressionCrossover
+##' @export
+crossoverexprFast <- function(expr1, expr2)
+  .Call("crossover_single_point_R", expr1, expr2)[[1]]
+
+##' @rdname expressionCrossover
+##' @export
+crossoverexprTwoPoint <- function(expr1, expr2) {
+  index1 <- floor(runif(1, 0, exprSize(expr1)))
+  index2 <- floor(runif(1, 0, exprSize(expr2)))
+  child1 <- replaceSubtreeAt(expr1, index1, subtreeAt(expr2, index2))
+  child2 <- replaceSubtreeAt(expr2, index2, subtreeAt(expr1, index1))
+  list(child1, child2)  
+}
+
 ##' @rdname expressionCrossover
 ##' @export
 crossoverTyped <- function(func1, func2, crossoverprob = 0.1,
                            breedingFitness = function(individual) TRUE,
                            breedingTries = 50) {
   doCrossoverTyped <- function() {
-    child <- new.function() 
+    child <- new.function(envir = environment(func1)) 
     formals(child) <- formals(func1)
     body(child) <- crossoverexprTyped(body(func1), body(func2), crossoverprob)
     child
